use std::collections::HashSet;

use jid::JID;
use sqlx::Sqlite;

pub enum ContactUpdate {
    Name(Option<String>),
    AddToGroup(String),
    RemoveFromGroup(String),
}

#[derive(Debug, sqlx::FromRow)]
pub struct Contact {
    // jid is the id used to reference everything, but not the primary key
    pub user_jid: JID,
    pub subscription: Subscription,
    /// client user defined name
    pub name: Option<String>,
    // TODO: avatar, nickname
    /// nickname picked by contact
    // nickname: Option<String>,
    #[sqlx(skip)]
    pub groups: HashSet<String>,
}

#[derive(Debug)]
enum Subscription {
    None,
    PendingOut,
    PendingIn,
    OnlyOut,
    OnlyIn,
    OutPendingIn,
    InPendingOut,
    Buddy,
    // TODO: perhaps don't need, just emit event to remove contact
    // Remove,
}

impl sqlx::Type<Sqlite> for Subscription {
    fn type_info() -> <Sqlite as sqlx::Database>::TypeInfo {
        <&str as sqlx::Type<Sqlite>>::type_info()
    }
}

impl sqlx::Decode<'_, Sqlite> for Subscription {
    fn decode(
        value: <Sqlite as sqlx::Database>::ValueRef<'_>,
    ) -> Result<Self, sqlx::error::BoxDynError> {
        let value = <&str as sqlx::Decode<Sqlite>>::decode(value)?;
        match value {
            "none" => Ok(Self::None),
            "pending-out" => Ok(Self::PendingOut),
            "pending-in" => Ok(Self::PendingIn),
            "only-out" => Ok(Self::OnlyOut),
            "only-in" => Ok(Self::OnlyIn),
            "out-pending-in" => Ok(Self::OutPendingIn),
            "in-pending-out" => Ok(Self::InPendingOut),
            "buddy" => Ok(Self::Buddy),
            _ => unreachable!(),
        }
    }
}

impl sqlx::Encode<'_, Sqlite> for Subscription {
    fn encode_by_ref(
        &self,
        buf: &mut <Sqlite as sqlx::Database>::ArgumentBuffer<'_>,
    ) -> Result<sqlx::encode::IsNull, sqlx::error::BoxDynError> {
        let value = match self {
            Subscription::None => "none",
            Subscription::PendingOut => "pending-out",
            Subscription::PendingIn => "pending-in",
            Subscription::OnlyOut => "only-out",
            Subscription::OnlyIn => "only-in",
            Subscription::OutPendingIn => "out-pending-in",
            Subscription::InPendingOut => "in-pending-out",
            Subscription::Buddy => "buddy",
        };
        <&str as sqlx::Encode<Sqlite>>::encode(value, buf)
    }
}
